require 'timeout'
require 'json'
require 'cz/ues/platform/process/UESFuture'

extClass = Ues::Platform::Command::UESCommand

class << extClass
  alias old_exec exec
  alias old_convert_to_ruby convert_to_ruby
end

def extClass.exec(mainEntityUri, parameters = nil)
  use_case_code = nil
  if parameters.java_kind_of?(Hash)
    params = parameters[:parameters]
    if params.java_kind_of?(Hash) || params.java_kind_of?(Java::java.util.Map)
      parameters[:parameters] = Java::java.util.HashMap.new(params)
    end
    use_case_code = parameters[:useCaseCode]
  elsif parameters.java_kind_of?(Java::cz.ues.platform.command.dto.UESCommandExec)
    params = parameters.parameters
    if params.java_kind_of?(Hash) || params.java_kind_of?(Java::java.util.Map)
      parameters.parameters = Java::java.util.HashMap.new(params)
    end
    use_case_code = parameters.getUseCaseCode
  end
  result = self.old_exec(mainEntityUri, parameters)
  if (result.java_kind_of?(Hash) || result.java_kind_of?(Java::java.util.Map)) && result.size == 3 && result['future'] && result.has_key?('taskUri') && result.has_key?('execUrl')
    # TODO This branch represents deprecated way of asynchronous command handling.
    # New implementation relies on invocation of "state" action directly on command
    # controller (bypassing problem with common user having no access to operational
    # territory where asynchronous task is processed).
    # Return Future instead of result Hash.
    parts = result['taskUri'].split(':')
    result_code = parts.delete_at(-1)
    process_uri = parts.join(':')
    future = Ues::Platform::Process::UESFuture.new(process_uri, result_code)
    # Since commands v2 may use different process storage, future must be patched
    # to reliably provide command result.
    future.instance_variable_set(:@baseUrl, result['execUrl'])
    class << future
      def get(resultCode = nil, timeout = 60)
        task_uri = Java::cz.ues.platform.commons.uri.UESURI.parseURI("#{@processUri}#{@defaultResultCode}")
        client = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.command.spi.CommonCommandClient.java_class)
        dto = Java::cz.ues.platform.command.internal.dto.UESCommandInvoke.new
        dto.setBaseUrl(@baseUrl)
        dto.setAppUrlCode("ues/wcp")
        dto.setCmdUrlPath("ues/platform/command/UESAsyncCommandExecutor/getState")
        Timeout::timeout(timeout) do
          state = client.invoke(task_uri, dto)
          wait = 0.5
          while state['state'] == 'WAITING' || state['state'] == 'RUNNING' do
            wait *= 2
            sleep(wait)
            state = client.invoke(task_uri, dto)
          end
          if state['state'] == 'ERROR'
            if state['exception'].nil? || state['exception'] !~ /^{.*}$/
              raise UESProcessExplorerRTException.new(UESProcessExplorerRTException::E05P02::M16,
                "Asynchronous process was not successfully finished:\n\tE05100.MNA: Unexpected error")              
            else
              error = JSON.parse(state['exception'], :symbolize_names => true)
              raise UESProcessExplorerRTException.new(UESProcessExplorerRTException::E05P02::M16,
                "Asynchronous process was not successfully finished:\n\t{0}#{error[:code]}: #{error[:message]}") 
            end
          else
            return state['result']
          end
        end
      end
    end
    return future
  elsif (result.java_kind_of?(Hash) || result.java_kind_of?(Java::java.util.Map)) && result.size == 6 && result['future'] &&
    result.has_key?('taskUri') && result.has_key?('appUrlCode') && result.has_key?('cmdUrlPath') && result.has_key?('action') && result.has_key?('uuUri')
    # Return Future instead of result Hash.
    parts = result['taskUri'].split(':')
    result_code = parts.delete_at(-1)
    process_uri = parts.join(':')
    future = Ues::Platform::Process::UESFuture.new(process_uri, result_code)
    # Since commands v2 may use different process storage, future must be patched
    # to reliably provide command result.
    future.instance_variable_set(:@appUrlCode, result['appUrlCode'])
    future.instance_variable_set(:@cmdUrlPath, result['cmdUrlPath'])
    future.instance_variable_set(:@action, result['action'])
    future.instance_variable_set(:@mainEntityUri, result['uuUri'])
    class << future
      def get(resultCode = nil, timeout = 60)
        task_uri = Java::cz.ues.platform.commons.uri.UESURI.parseURI("#{@processUri}#{@defaultResultCode}")
        client = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.command.spi.CommonCommandClient.java_class)
        dto = Java::cz.ues.platform.command.internal.dto.UESCommandInvoke.new
        dto.setAppUrlCode(@appUrlCode)
        dto.setCmdUrlPath(@cmdUrlPath)
        dto.setAction("{@action}AsyncStatus")
        params = Java::java.util.HashMap.new()
        params.put('taskUri', task_uri)
        dto.setParameters(params)
        Timeout::timeout(timeout) do
          state = client.invoke(@mainEntityUri, dto)
          wait = 0.5
          while state['state'] == 'WAITING' || state['state'] == 'RUNNING' do
            wait *= 2
            sleep(wait)
            state = client.invoke(@mainEntityUri, dto)
          end
          if state['state'] == 'ERROR'
            if state['exception'].nil? || state['exception'] !~ /^{.*}$/
              raise UESProcessExplorerRTException.new(UESProcessExplorerRTException::E05P02::M16,
                "Asynchronous process was not successfully finished:\n\tE05100.MNA: Unexpected error")              
            else
              error = JSON.parse(state['exception'], :symbolize_names => true)
              raise UESProcessExplorerRTException.new(UESProcessExplorerRTException::E05P02::M16,
                "Asynchronous process was not successfully finished:\n\t{0}#{error[:code]}: #{error[:message]}") 
            end
          else
            return state['result']
          end
        end
      end
    end
    return future
  else
    return result
  end
end

def extClass.convert_to_ruby(value) #:nodoc:
  # Java map is by default not supported by conversions.
  if value.java_kind_of?(Java::java.util.Map)
    result = {}
    value.each do |key, val|
      result[key] = convert_to_ruby(val)
    end
    return result
  else
    return old_convert_to_ruby(value)
  end
end
