class Ues::Platform::Commons::Query::UESQueryHelper

  # Formats parameters values for the UES query string. It provides a mechanism
  # for creating the right string representation of objects such as dates etc.
  # <br>
  # example: Input string queryStub = "dateFrom > ?" and 1 parameter =
  # Date(December 31th, 2010 at 11 pm) are formatted into the string
  # "dateFrom > '2010-12-31T23:00'" which conforms the UES query syntax. <br>
  # Note that it is not permitted to use placeholder '?' for other purposes than
  # parameters values binding.
  # 
  # <tt><b>param queryStub</b></tt> ues query string optionally containing character '?' as a
  #          placeholder for the query parameters value.
  # <tt><b>param parameters</b></tt> Objects containing values of parameters. There need to be
  #          an object for every placeholder '?' in stub. Only java String,
  #          Boolean, Long/Integer/Short/Byte and Date classes are supported.
  # <tt><b>return</b></tt> UES query string with parameters bound.
  # <tt><b>since</b></tt> UES6-10
  #
  def self.bindParameters(queryStub, *parameters)
    #".to_java" below is crucial because it converts ruby collection to java collection
    Java::cz.ues.platform.commons.query.UESQueryHelper.bindParameters(queryStub, parameters.to_java)
  end

end
